import fs, { Dirent } from 'fs';
import fsPromises from 'fs/promises';
export { chmodSync, mkdirSync, renameSync, rmdirSync, rmSync, statSync, lstatSync, unlinkSync, } from 'fs';
export declare const readdirSync: (path: fs.PathLike) => Dirent[];
export declare const promises: {
    chmod: typeof fsPromises.chmod;
    mkdir: typeof fsPromises.mkdir;
    readdir: (path: fs.PathLike) => Promise<fs.Dirent<string>[]>;
    rename: typeof fsPromises.rename;
    rm: typeof fsPromises.rm;
    rmdir: typeof fsPromises.rmdir;
    stat: typeof fsPromises.stat;
    lstat: typeof fsPromises.lstat;
    unlink: typeof fsPromises.unlink;
};
//# sourceMappingURL=fs.d.ts.map