export declare const ignoreENOENT: <T>(p: Promise<T>, rethrow?: unknown) => Promise<void | T>;
export declare const ignoreENOENTSync: <T>(fn: () => T, rethrow?: unknown) => T | undefined;
//# sourceMappingURL=ignore-enoent.d.ts.map