export declare const balanced: (a: string | RegExp, b: string | RegExp, str: string) => false | {
    start: number;
    end: number;
    pre: string;
    body: string;
    post: string;
} | undefined;
export declare const range: (a: string, b: string, str: string) => undefined | [number, number];
//# sourceMappingURL=index.d.ts.map