export declare const isFsError: (o: unknown) => o is NodeJS.ErrnoException & {
    code: string;
    path: string;
};
export declare const errorCode: (er: unknown) => unknown;
//# sourceMappingURL=error.d.ts.map